import django_filters
from django.utils.translation import ugettext_lazy as _

I18n_ORDERING = _("Ordering")


class CrispyFilterMixin(object):
    form_class = 'form-inline'

    @property
    def form(self):
        from crispy_forms.helper import FormHelper
        from crispy_forms.layout import Submit
        self._form = super(CrispyFilterMixin, self).form
        self._form.helper = FormHelper(self._form)
        if self.form_class:
            self._form.helper.form_class = 'form-inline'
        self._form.helper.form_method = 'get'
        self._form.helper.layout.append(Submit('filter', _('Filter')))
        return self._form


class AutocompleteChoiceFilter(django_filters.ModelChoiceFilter):

    def __init__(self, autocomplete_name, *args, **kwargs):
        from autocomplete_light import shortcuts as autocomplete_light
        autocomplete = autocomplete_light.registry.get_autocomplete_from_arg(autocomplete_name)
        if 'label' not in kwargs:
            kwargs['label'] = autocomplete.model._meta.verbose_name
        kwargs['queryset'] = autocomplete.choices
        kwargs['widget'] = autocomplete_light.ChoiceWidget(autocomplete)
        super(AutocompleteChoiceFilter, self).__init__(*args, **kwargs)


class UserKwargFilterSetMixin(object):

    def __init__(self, user, *args, **kwargs):
        self.user = user
        super(UserKwargFilterSetMixin, self).__init__(*args, **kwargs)



from django import forms

from tinycontent.models import TinyContent


class GIODOMixin(object):
    def __init__(self, *args, **kwargs):
        super(GIODOMixin, self).__init__(*args, **kwargs)
        self.fields['giodo'] = forms.BooleanField(required=True)
        try:
            self.fields['giodo'].label = TinyContent.get_content_by_name('giodo').content
        except TinyContent.DoesNotExist:
            self.fields['giodo'].label = 'Lorem ipsum'



from guardian.shortcuts import assign_perm
from django.core.exceptions import ImproperlyConfigured


class PermissionStatusMixin(object):
    """Mixin to verify object permission status codes for different users

    Require user with username='john' and password='pass'

    Attributes:
        permission (list): Description
        status_anonymous (int): Status code for anonymouser
        status_has_permission (int): Status code for user with permission
        status_no_permission (403): Status code for user without permission
        url (string): url to test
    """
    url = None
    permission = None
    status_anonymous = 302
    status_no_permission = 403
    status_has_permission = 200

    def setUp(self):
        super(PermissionStatusMixin, self).setUp()

    def get_url(self):
        """Get url to tests

        Returns:
            str: url to test

        Raises:
            ImproperlyConfigured: Missing a url to test
        """
        if self.url is None:
            raise ImproperlyConfigured(
                '{0} is missing a url to test. Define {0}.url '
                'or override {0}.get_url().'.format(self.__class__.__name__))
        return self.url

    def get_permission(self):
        """Returns the permission to assign for granted permission user

        Returns:
            list: A list of permission in format ```codename.permission_name```

        Raises:
            ImproperlyConfigured: Missing a permission to assign
        """
        if self.permission is None:
            raise ImproperlyConfigured(
                '{0} is missing a permissions to assign. Define {0}.permission '
                'or override {0}.get_permission().'.format(self.__class__.__name__))
        return self.permission

    def get_permission_object(self):
        """Returns object of permission-carrying object for grant permission
        """
        return getattr(self, 'permission_object', None)

    def grant_permission(self):
        """Grant permission to user in self.user

        Returns:
            TYPE: Description
        """
        for perm in self.get_permission():
            obj = self.get_permission_object()
            assign_perm(perm, self.user, obj)

    def login_permitted_user(self):
        """Login client to user with granted permissions

        """
        self.grant_permission()
        self.client.login(username='john', password='pass')

    def test_status_code_for_anonymous_user(self):
        """A test status code of response for anonymous user

        """
        response = self.client.get(self.get_url())
        self.assertEqual(response.status_code, self.status_anonymous)

    def test_status_code_for_signed_user(self):
        """A test for status code of response for signed (logged-in) user

        Only login before test.
        """
        self.client.login(username='john', password='pass')
        response = self.client.get(self.get_url())
        self.assertEqual(response.status_code, self.status_no_permission)

    def test_status_code_for_privileged_user(self):
        """A test for status code of response for privileged user

        Grant permission to permission-carrying object and login before test
        """
        self.grant_permission()
        self.client.login(username='john', password='pass')
        response = self.client.get(self.get_url())
        self.assertEqual(response.status_code, self.status_has_permission)



from django.core.exceptions import ImproperlyConfigured
from django.urls import reverse


class AdminTestCaseMixin(object):
    user_factory_cls = None
    factory_cls = None
    model = None
    changelist_viewname = None
    change_viewname = None
    delete_viewname = None
    history_viewname = None
    QUERY_LIMIT = 30

    def get_changelist_viewname(self):
        if self.changelist_viewname is None and self.model is None:
            raise ImproperlyConfigured('{0} is missing a {0}.changelist_viewname or {0}.model. Define '
                                       '{0}.changelist_viewname or {0}.model or override '
                                       '{0}.get_changelist_viewname().'.format(self.__class__.__name__))
        return self.changelist_viewname or 'admin:{}_{}_changelist'.format(self.model._meta.app_label,
                                                                           self.model._meta.model_name)

    def get_change_viewname(self):
        if self.change_viewname is None and self.model is None:
            raise ImproperlyConfigured('{0} is missing a {0}.change_viewname or {0}.model. Define '
                                       '{0}.change_viewname or {0}.model or override '
                                       '{0}.get_change_viewname().'.format(self.__class__.__name__))
        return self.changelist_viewname or 'admin:{}_{}_change'.format(self.model._meta.app_label,
                                                                       self.model._meta.model_name)

    def get_delete_viewname(self):
        if self.delete_viewname is None and self.model is None:
            raise ImproperlyConfigured('{0} is missing a {0}.changelist_viewname or {0}.model. Define '
                                       '{0}.change_viewname or {0}.model or override '
                                       '{0}.get_delete_viewname().'.format(self.__class__.__name__))
        return self.delete_viewname or 'admin:{}_{}_delete'.format(self.model._meta.app_label,
                                                                   self.model._meta.model_name)

    def get_history_viewname(self):
        if self.history_viewname is None and self.model is None:
            raise ImproperlyConfigured('{0} is missing a {0}.history_viewname or {0}.model. Define '
                                       '{0}.history_viewname or {0}.model or override '
                                       '{0}.get_history_viewname().'.format(self.__class__.__name__))
        return self.history_viewname or 'admin:{}_{}_history'.format(self.model._meta.app_label,
                                                                     self.model._meta.model_name)

    def get_factory_cls(self):
        if self.factory_cls is None:
            raise ImproperlyConfigured('{0} is missing a {0}.factory_cls. Define {0}.factory_cls or override '
                                       '{0}.get_factory_cls().'.format(self.__class__.__name__))
        return self.factory_cls

    def setUp(self):
        self.object = self.factory_cls()
        self.user = self.user_factory_cls(is_superuser=True,
                                          is_staff=True)
        self.client.login(username=self.user, password='pass')

    def test_status_changelist(self):
        url = reverse(self.get_changelist_viewname())
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)

    def test_status_change_view(self):
        url = reverse(self.get_change_viewname(), args=[self.object.pk])
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)

    def test_status_delete_view(self):
        url = reverse(self.get_delete_viewname(), args=[self.object.pk])
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)

    def test_status_history_view(self):
        url = reverse(self.get_history_viewname(), args=[self.object.pk])
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)

    def test_changelist_queries_limit(self):
        self.factory_cls.create_batch(size=20)
        url = reverse(self.get_changelist_viewname())
        with self.assertNumQueriesLessThan(self.QUERY_LIMIT):
            self.client.get(url)

    def test_change_view_queries_limit(self):
        self.factory_cls.create_batch(size=20)
        url = reverse(self.get_change_viewname(), args=[self.object.pk])
        with self.assertNumQueriesLessThan(self.QUERY_LIMIT):
            self.client.get(url)



from functools import partial

from django import forms


class PartialMixin(object):

    @classmethod
    def partial(cls, *args, **kwargs):
        return partial(cls, *args, **kwargs)


class AuthorMixin(object):
    def save(self, *args, **kwargs):
        if self.instance.pk:
            self.instance.modified_by = self.user
        else:
            self.instance.created_by = self.user
        return super(AuthorMixin, self).save(*args, **kwargs)



import django_filters
from autocomplete_light import shortcuts as autocomplete_light


class AutocompleteChoiceFilter(django_filters.ModelChoiceFilter):
    def __init__(self, autocomplete_name, *args, **kwargs):
        autocomplete = autocomplete_light.registry.get_autocomplete_from_arg(autocomplete_name)
        kwargs['queryset'] = autocomplete.choices
        kwargs['widget'] = autocomplete_light.ChoiceWidget(autocomplete)
        super(AutocompleteChoiceFilter, self).__init__(*args, **kwargs)



from __future__ import absolute_import
from crispy_forms.helper import FormHelper
from crispy_forms.layout import Reset, Submit
from django.forms import BaseFormSet
from django.forms.models import BaseInlineFormSet
from django.utils.translation import ugettext as _
from django.utils.translation import ugettext_lazy as _l


class FormsetHelper(FormHelper):
    form_tag = False
    form_method = 'post'


class TableFormSetHelper(FormsetHelper):
    template = 'bootstrap/table_inline_formset.html'


class TableFormSetMixin(object):
    def __init__(self, *args, **kwargs):
        super(InlineTableFormSet, self).__init__(*args, **kwargs)
        self.helper = TableFormSetHelper()


class InlineTableFormSet(TableFormSetMixin, BaseInlineFormSet):
    pass


class TableFormSet(TableFormSetMixin, BaseFormSet):
    pass


class BaseTableFormSet(BaseInlineFormSet):
    def __init__(self, *args, **kwargs):
        super(BaseTableFormSet, self).__init__(*args, **kwargs)
        self.helper = TableFormSetHelper()


class HelperMixin(object):

    form_helper_cls = FormHelper

    def __init__(self, *args, **kwargs):
        super(HelperMixin, self).__init__(*args, **kwargs)
        self.helper = getattr(self, 'helper', self.form_helper_cls(self))


class SingleButtonMixin(HelperMixin):
    """Dynamically add crispy button to form layout

    Example
    -------
    Usage of mixins is obvious::

        from django import forms
        from atom.ext.crispy_forms.forms import SingleButtonMixin

        class PersonModelForm(SingleButtonMixin, forms.ModelForm):

            class Meta:
                model = Person

    Extends
    -------
    HelperMixin
    """

    @property
    def action_text(self):
        """A text of added action submit button.

        In standards it detects when forms save or update objects

        Returns
        -------
        string -- A text used in button
        """
        return _('Update') if hasattr(self, 'instance') and self.instance.pk else _('Save')

    def __init__(self, *args, **kwargs):
        super(SingleButtonMixin, self).__init__(*args, **kwargs)
        self.helper.add_input(Submit('action', self.action_text, css_class="btn-primary"))


class FormHorizontalMixin(HelperMixin):
    def __init__(self, *args, **kwargs):
        super(FormHorizontalMixin, self).__init__(*args, **kwargs)
        self.helper.form_class = 'form-horizontal'
        self.helper.label_class = 'col-lg-3'
        self.helper.field_class = 'col-lg-9'



from django.contrib import messages
from django.core.exceptions import ImproperlyConfigured
from django.http import HttpResponseRedirect
from django.utils.encoding import force_text
from django.utils.translation import ugettext as _
from django.views.generic.detail import (BaseDetailView,
                                         SingleObjectTemplateResponseMixin)


class FormInitialMixin(object):

    def get_initial(self, *args, **kwargs):
        initial = super(FormInitialMixin, self).get_initial(*args, **kwargs)
        initial.update(self.request.GET.dict())
        return initial


class MessageMixin(object):
    success_message = None

    def get_success_message(self):
        if self.success_message is None:
            raise NotImplementedError("Provide success_message or get_success_message")
        return self.success_message.format(**self.object.__dict__)


class DeleteMessageMixin(object):
    hide_field = None

    def get_success_message(self):
        template = dict(object=self.object, verbose_name=self.model._meta.verbose_name)
        return _("{verbose_name} {object} deleted!").format(**template)

    def delete(self, request, *args, **kwargs):
        self.object = self.get_object()
        success_url = self.get_success_url()
        if self.hide_field:
            setattr(self.object, self.hide_field, False)
            self.object.save()
        else:
            self.object.delete()
        messages.add_message(request, messages.SUCCESS, self.get_success_message())
        return HttpResponseRedirect(success_url)


class ActionMixin(object):
    success_url = None

    def action(self):
        raise ImproperlyConfigured("No action to do. Provide a action body.")

    def post(self, request, *args, **kwargs):
        self.object = self.get_object()
        success_url = self.get_success_url()
        self.action()
        return HttpResponseRedirect(success_url)

    def get_success_url(self):
        if self.success_url:
            self.success_url = force_text(self.success_url)
            return self.success_url.format(**self.object.__dict__)
        else:
            raise ImproperlyConfigured(
                "No URL to redirect to. Provide a success_url.")


class BaseActionView(ActionMixin, BaseDetailView):
    """
    Base view for action on an object.
    Using this base class requires subclassing to provide a response mixin.
    """


class ActionView(SingleObjectTemplateResponseMixin, BaseActionView):
    template_name_suffix = '_action'


class ActionMessageMixin(MessageMixin):

    def post(self, request, *args, **kwargs):
        response = super(ActionMessageMixin, self).post(request, *args, **kwargs)
        messages.add_message(request, messages.SUCCESS, self.get_success_message())
        return response


class CreateMessageMixin(object):

    def get_form_valid_message(self):
        template = dict(object=self.object, verbose_name=self.model._meta.verbose_name)
        return _("{verbose_name} {object} created!").format(**template)


class UpdateMessageMixin(object):

    def get_form_valid_message(self):
        template = dict(object=self.object, verbose_name=self.model._meta.verbose_name)
        return _("{verbose_name} {object} updated!").format(**template)



from __future__ import absolute_import

from django.contrib import messages
from django.forms.models import inlineformset_factory
from django.http import HttpResponseRedirect
from django.utils.translation import ugettext as _

from .forms import BaseTableFormSet


class FormSetMixin(object):
    inline_model = None
    inline_form_cls = None
    formset_cls = BaseTableFormSet
    formset = None  # precomputed by default

    def get_instance(self):
        return None

    def get_success_url(self):
        return self.object.get_absolute_url()

    def get_formset(self):
        return self.formset or inlineformset_factory(self.model, self.inline_model,
                                                     form=self.inline_form_cls,
                                                     formset=self.formset_cls)

    def get_context_data(self, **kwargs):
        context = super(FormSetMixin, self).get_context_data(**kwargs)
        context.update({'formset': self.get_formset()(instance=self.get_instance())})
        return context

    def get_formset_valid_message(self):
        return _("{0} created!").format(self.object)

    def get_form(self, *args, **kwargs):
        form = super(FormSetMixin, self).get_form(*args, **kwargs)
        if hasattr(form, 'helper'):
            form.helper.form_tag = False
        return form

    def formset_valid(self, form, formset):
        formset.save()
        messages.success(self.request, self.get_formset_valid_message())
        return HttpResponseRedirect(self.get_success_url())

    def formset_invalid(self, form, formset):
        return self.render_to_response(self.get_context_data(form=form, formset=formset))

    def get_formset_kwargs(self):
        return {}

    def form_valid(self, form):
        self.object = form.save(commit=False)
        FormSet = self.get_formset()
        formset = FormSet(self.request.POST or None,
                          self.request.FILES,
                          instance=self.object,
                          **self.get_formset_kwargs())

        if formset.is_valid():
            self.object.save()
            return self.formset_valid(form, formset)
        else:
            return self.formset_invalid(form, formset)



class UserKwargFilterSetMixin(object):

    def get_filterset_kwargs(self, *args, **kwargs):
        kwargs = super(UserKwargFilterSetMixin, self).get_filterset_kwargs(*args, **kwargs)
        kwargs['user'] = self.request.user
        return kwargs



from django.utils.translation import ugettext as _
from guardian.forms import UserObjectPermissionsForm


class PermissionsTranslationMixin(object):
    def __init__(self, *args, **kwargs):
        super(PermissionsTranslationMixin, self).__init__(*args, **kwargs)
        choices = [(key, _(value)) for key, value in self.fields['permissions'].choices]
        self.fields['permissions'].choices = choices


class TranslatedUserObjectPermissionsForm(PermissionsTranslationMixin, UserObjectPermissionsForm):
    pass



from django import template

register = template.Library()


@register.simple_tag(takes_context=True)
def query_update(context, **kwargs):
    updated = context['request'].GET.copy()
    for k, v in kwargs.items():
        updated[k] = v
    return updated.urlencode()


@register.simple_tag(takes_context=True)
def query_append(context, k=None, v=None, **kwargs):
    if k and v:
        kwargs[k] = v
    updated = context['request'].GET.copy()
    for k, v in kwargs.items():
        updated.appendlist(k, v)
    return updated.urlencode()


@register.simple_tag
def test(test):
    import ipdb
    ipdb.set_trace()



from os.path import basename

from django.db import models
from django.utils.translation import ugettext_lazy as _


class AttachmentBase(models.Model):
    attachment = models.FileField(upload_to="letters/%Y/%m/%d", verbose_name=_("File"))

    @property
    def filename(self):
        return basename(self.attachment.name)

    def __unicode__(self):
        return "%s" % (self.filename)

    def get_absolute_url(self):
        return self.attachment.url

    class Meta:
        abstract = True
        verbose_name = _('Attachment')
        verbose_name_plural = _('Attachments')



from __future__ import absolute_import
from crispy_forms.helper import FormHelper
from crispy_forms.layout import Submit
from django.utils.translation import ugettext_lazy as _


class CrispyFilterMixin(object):
    form_class = 'form-inline'

    @property
    def form(self):
        self._form = super(CrispyFilterMixin, self).form
        self._form.helper = FormHelper(self._form)
        if self.form_class:
            self._form.helper.form_class = 'form-inline'
        self._form.helper.form_method = 'get'
        self._form.helper.layout.append(Submit('filter', _('Filter')))
        return self._form



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys
from setuptools import setup

version = '0.14.0'

if sys.argv[-1] == 'publish':
    os.system('python setup.py sdist upload')
    os.system('python setup.py bdist_wheel upload')
    sys.exit()

if sys.argv[-1] == 'tag':
    print("Tagging the version on github:")
    os.system("git tag -a %s -m 'version %s'" % (version, version))
    os.system("git push --tags")
    sys.exit()

readme = open('README.rst').read()
history = open('HISTORY.rst').read().replace('.. :changelog:', '')

setup(
    name='django-atom',
    version=version,
    description="""A different stuff for Django to faster make a world a better place.""",
    long_description=readme + '\n\n' + history,
    author='Adam Dobrawy',
    author_email='naczelnik@jawnosc.tk',
    url='https://github.com/ad-m/django-atom',
    packages=[
        'atom',
    ],
    include_package_data=True,
    install_requires=[
    ],
    use_2to3=True,
    license="BSD",
    zip_safe=False,
    keywords='django-atom',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
    ],
)
